/* Created by Michael Hhle <hoehle@dina.kvl.dk>, 14 Aug 2001 */

package org.omegahat.Simulation.MCMC.Listeners;
import org.omegahat.Simulation.MCMC.*;

import java.util.*;


/** 
 * This object serves as a proxy <code>MCMCListener</code> that discards
 * the first <code>burnin</code> events but then truthfully forwards to its
 * subscribers 
 **/

public class BurninProxyListener implements NotifyingObject, MCMCListener
{
    
    
protected Hashtable listeners = new Hashtable();  

protected int burninNumber = 1;
protected int totalEvents = 0;
    

public int getTotalEvents() { return totalEvents; }
public void resetTotalEvents() { totalEvents=0; }

public int getBurninNumber() { return burninNumber; }
public int setBurninNumber( int number ) 
{ 
    if(number<1) 
        throw new RuntimeException("Attempt to set burnin number to value less that 1");

    return burninNumber = number; 
}


    

    public class MyHandle implements MCMCListenerHandle
    {
        public MyHandle() {};
    }

    // we can always register and unregister listeners
    public MCMCListenerHandle registerListener  ( MCMCListener  listener ) 
    { 

        MCMCListenerHandle handle = new MyHandle();

        listeners.put( handle,  listener); 
        return handle;
    }
    public void    unregisterListener( MCMCListenerHandle handle   )
    { 
        listeners.remove( handle );
    }
    
    protected void   notifyAll( MCMCEvent e )
     {
       MCMCListener l = null;
    
       Enumeration iterator = listeners.elements();
       while( iterator.hasMoreElements() )
       {   
          l = (MCMCListener) iterator.nextElement();
          if( l != null) l.notify( e );
       }
     }


/** Function to be called for notification */
public void notify( MCMCEvent event )
{
    totalEvents++;

    // Discard the first burnin events.
    if( totalEvents > burninNumber)
       notifyAll(event);
}

public BurninProxyListener()
{
}

public BurninProxyListener(int number)
{
    burninNumber = number;
}

}

